/*
 * DCF_Debugging
 *
 * Created: 19.09.2016 17:57:22
 * Author : devpap
 */ 

#include "uart.h"

#include <avr/io.h>
#include <avr/interrupt.h>

#include <stdlib.h>
#include <stdio.h>

volatile uint16_t g_millis; /* Zhler fr Millisekunden der kurzen Signale 0 oder 1*/
volatile uint16_t g_breakMillis; /* Zhler fr Millisekunden der Signalpause */
volatile uint8_t g_dcfInterrupt; /* Flag das angibt, dass ein Signal aktiv ist */

/**
 * @brief Initialisiert den Timer2 auf dem Atmega8 im CTC Modus mit dem Prescaler 128. Die ISR wird jede Millisekunde aufgerufen.
 * @param - 
 * @return - 
 */
void timer2_init()
{
	uart_puts_nl("Init Timer2");
	TCCR2 |= (1 << WGM21) | (1 << CS22) | (1 << CS20);
	OCR2 = 0x7C;
	TIMSK |= (1 << OCIE2);
	g_millis = g_breakMillis = 0;
	g_dcfInterrupt = 0;
}

/**
 * @brief Aktiviert den Interrupt an Pin4
 * @param - 
 * @return - 
 */
void dcf_interrupt0_init()
{
	uart_puts_nl("Init dcf_interrupt0");
	GICR |= (1 << INT0);
	MCUCR |= (1 << ISC01) | (1 << ISC00);
}

/**
 * @brief Zurcksetzen aller Status und Interrupt an Pin 4 wieder aktivieren
 * @param - 
 * @return - 
 */
void reinit()
{
	g_dcfInterrupt = 0;
	g_millis = 0;
	g_breakMillis = 0;
	GICR |= (1 << INT0);
}

/* Interrupt0 Pin 4*/
ISR(INT0_vect)
{
	/* Externen Interrupt zwischenzeitlich deaktiveren. Wird wieder aktiviert, wenn Signal verarbeitet wurde*/
	GICR &= ~(1 << INT0);
	PORTB |= (1 << PB1);
	TCNT2 = 0x00;
	g_millis = 0;
	g_dcfInterrupt = 1;
}

/**
 * @brief Ausgelt jede Millisekunde
 * @param - 
 * @return - 
 */
ISR(TIMER2_COMP_vect)
{
	if (g_dcfInterrupt)
	{
		if(PIND & (1 << PD2))
		{/* Pin abfragen */
			++g_millis;
		}	
		else
		{
			char time[5];
			if(g_breakMillis > 1500)
			{/* Lange Pause, bevor das neue Signal beginnt */
				itoa(g_breakMillis, time, 10);
				uart_puts_nl(time);
				reinit();
				return;
			}
			else
			{/* Hier muss der Impuls fr eine 0 oder eine 1 verarbeitet werden */
				itoa(g_millis, time, 10);
				uart_puts_nl(time);
				PORTB &= ~(1 << PB1);
				reinit();
			}
		}
	}
	else
	{
		++g_breakMillis;
	}
}

/**
 * @brief Los
 */
int main(void)
{
	uart_init();
	uart_puts_nl("Programmstart");
	dcf_interrupt0_init();
	timer2_init();
	
	/* globale Interrupts */
	sei();
	
	/* Status LED, damit man was sieht */
	DDRB = 0x03;
	PORTB |= (0 << PB1) | (1 << PB0);
	    
    while (1) 
    {
    }
}

