/*
 * uart.c
 *
 * Created: 24.04.2016 19:17:39
 * Author : devpap
 * Brief  : Rudimentre Implementierung fr die Kommunikation ber UART. Kein UART ber Interrupt, da sich die Interrupts mit
 *			mit dem DCF Modul und den Timer blockiert haben.
 */ 

#include <avr/io.h>

/* Definitionen fr UART Kommunikation */
#define F_CPU 16000000L
#define UART_BAUD_RATE 4800UL

/* Macros berechnen, ob der Fehler fuer die Taktfrequenz mit der geforderten Baudrate noch unter 1% Fehler liegt. Kompilieren erst mglich, wenn dies zutrifft */
/* cleveres Runden */
#define UBRR_VAL ((F_CPU+UART_BAUD_RATE*8)/(UART_BAUD_RATE*16)-1)
/* Reale Baudrate */
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))
/* Fehler in Promille, 1000 = kein Fehler. */
#define BAUD_ERROR ((BAUD_REAL*1000)/UART_BAUD_RATE)

#if ((BAUD_ERROR<990) || (BAUD_ERROR>1010))
#error Systematischer Fehler der Baudrate groesser 1% und damit zu hoch!
#endif

/* @bief Initialisierung der UART Kommunikation Pin3 (TXD)
 * @param -
 * @return -
 */
void uart_init()
{
	volatile uint16_t baudvalue = UBRR_VAL;
	UBRRH = (uint8_t)(baudvalue>>8);
	UBRRL = (uint8_t) baudvalue;
	/* Senden ermglichen */
	UCSRB |= (1<<TXEN);
	/* Asynchron 8N1 => 8 Datenbits, N (No Parity) kein Parittsbit , 1 Stopbit */
	UCSRC = (1<<URSEL) | (1<<UCSZ1) | (1<<UCSZ0);  // Asynchron 8N1
}

/* @brief Senden eines einzelnen Zeichens
 * @param zu sendendes Zeichen
 * @return -
 */
int uart_putc(unsigned char c)
{
	/* Warten bis Senden mglich ist, das Datenregistern muss leer sein*/
	while (!(UCSRA & (1<<UDRE)));
	/* Zeichen senden */
	UDR = c;
	return 0;
}

/* @brief Senden einer Zeichenkette
 * @param zu sendendes Zeichenkette
 * @return -
 */
void uart_puts (char *text)
{
	while (*text)
	{
		uart_putc(*text);
		++text;
	}
}

/* @brief Senden einer Zeichenkette mit anschlieendem Zeilenumbruch
 * @param zu sendendes Zeichenkette
 * @return -
 */
void uart_puts_nl (char *text)
{
	while (*text)
	{
		uart_putc(*text);
		++text;
	}
	uart_putc('\n');
}